

import unittest
from cpuinfo import *
import helpers




class TestParseCPUString(unittest.TestCase):
	def test_to_decimal_string(self):
		self.assertEqual('2.8', cpuinfo._to_decimal_string('2.80'))
		self.assertEqual('2.0', cpuinfo._to_decimal_string('2'))
		self.assertEqual('3.0', cpuinfo._to_decimal_string(3))
		self.assertEqual('6.5', cpuinfo._to_decimal_string(6.5))
		self.assertEqual('7.002', cpuinfo._to_decimal_string(7.002))
		self.assertEqual('4.00000000001', cpuinfo._to_decimal_string('4.00000000001'))
		self.assertEqual('5.0', cpuinfo._to_decimal_string('5.000000000000'))

		self.assertEqual('0.0', cpuinfo._to_decimal_string('invalid'))
		self.assertEqual('0.0', cpuinfo._to_decimal_string('8.778.9'))
		self.assertEqual('0.0', cpuinfo._to_decimal_string(''))
		self.assertEqual('0.0', cpuinfo._to_decimal_string(None))

	def test_hz_short_to_full(self):
		self.assertEqual((2800000000, 0), cpuinfo._hz_short_to_full('2.8', 9))
		self.assertEqual((1200000, 0), cpuinfo._hz_short_to_full('1.2', 6))
		self.assertEqual((3200000000, 0), cpuinfo._hz_short_to_full('3.2', 9))
		self.assertEqual((9001200000, 0), cpuinfo._hz_short_to_full('9001.2', 6))
		self.assertEqual((0, 0), cpuinfo._hz_short_to_full('0.0', 0))
		self.assertEqual((2, 87), cpuinfo._hz_short_to_full('2.87', 0))

		self.assertEqual((0, 0), cpuinfo._hz_short_to_full('invalid', 0))
		self.assertEqual((0, 0), cpuinfo._hz_short_to_full('8.778.9', 0))
		self.assertEqual((0, 0), cpuinfo._hz_short_to_full('', 0))
		self.assertEqual((0, 0), cpuinfo._hz_short_to_full(None, 0))

	def test_hz_friendly_to_full(self):
		self.assertEqual((2800000000, 0), cpuinfo._hz_friendly_to_full('2.80GHz'))
		self.assertEqual((1200000, 0), cpuinfo._hz_friendly_to_full('1.20 mHz'))
		self.assertEqual((3693150000, 0), cpuinfo._hz_friendly_to_full('3693.15-MHz'))
		self.assertEqual((12000000000, 0), cpuinfo._hz_friendly_to_full('12 GHz'))
		self.assertEqual((2, 6), cpuinfo._hz_friendly_to_full('2.6 Hz'))
		self.assertEqual((0, 0), cpuinfo._hz_friendly_to_full('0 Hz'))

		self.assertEqual((0, 0), cpuinfo._hz_friendly_to_full('invalid'))
		self.assertEqual((0, 0), cpuinfo._hz_friendly_to_full('8.778.9'))
		self.assertEqual((0, 0), cpuinfo._hz_friendly_to_full(''))
		self.assertEqual((0, 0), cpuinfo._hz_friendly_to_full(None))

	def test_hz_short_to_friendly(self):
		self.assertEqual('2.8000 GHz', cpuinfo._hz_short_to_friendly('2.8', 9))
		self.assertEqual('1.2000 MHz', cpuinfo._hz_short_to_friendly('1.2', 6))
		self.assertEqual('3.2000 GHz', cpuinfo._hz_short_to_friendly('3.2', 9))
		self.assertEqual('1.3000 Hz', cpuinfo._hz_short_to_friendly('1.3', 0))
		self.assertEqual('0.0000 Hz', cpuinfo._hz_short_to_friendly('0.0', 0))

		self.assertEqual('0.0000 Hz', cpuinfo._hz_short_to_friendly('invalid', 0))
		self.assertEqual('0.0000 Hz', cpuinfo._hz_short_to_friendly('8.778.9', 0))
		self.assertEqual('0.0000 Hz', cpuinfo._hz_short_to_friendly('', 0))
		self.assertEqual('0.0000 Hz', cpuinfo._hz_short_to_friendly(None, 0))

	def test_parse_cpu_brand_string(self):
		hz, scale = cpuinfo._parse_cpu_brand_string('Intel(R) Pentium(R) CPU G640 @ 2.80GHz')
		self.assertEqual((hz, scale), ('2.8', 9))

		hz, scale = cpuinfo._parse_cpu_brand_string('Intel(R) Pentium(R) CPU @ 1.20MHz')
		self.assertEqual((hz, scale), ('1.2', 6))

		# NOTE: No @ symbol
		hz, scale = cpuinfo._parse_cpu_brand_string('Intel(R) Pentium(R) D CPU 3.20GHz')
		self.assertEqual((hz, scale), ('3.2', 9))

		# NOTE: No @ symbol and no Hz
		hz, scale = cpuinfo._parse_cpu_brand_string('AMD Ryzen 7 2700X Eight-Core Processor')
		self.assertEqual((hz, scale), ('0.0', 0))

	def test_parse_cpu_brand_string_dx(self):
		hz, scale, brand, vendor_id, stepping, model, family = \
		cpuinfo._parse_cpu_brand_string_dx("Intel(R) Pentium(R) CPU G640 @ 2.80GHz (fam: 06, model: 2a, stepping: 07)")
		self.assertEqual('Intel(R) Pentium(R) CPU G640 @ 2.80GHz', brand)
		self.assertEqual((hz, scale), ('2.8', 9))
		self.assertEqual((vendor_id, stepping, model, family), (None, 7, 42, 6))

		hz, scale, brand, vendor_id, stepping, model, family = \
		cpuinfo._parse_cpu_brand_string_dx("Intel(R) Pentium(R) CPU G640 @ 2.80GHz (family: 0x6, model: 0x2a, stepping: 0x7)")
		self.assertEqual('Intel(R) Pentium(R) CPU G640 @ 2.80GHz', brand)
		self.assertEqual((hz, scale), ('2.8', 9))
		self.assertEqual((vendor_id, stepping, model, family), (None, 7, 42, 6))

		hz, scale, brand, vendor_id, stepping, model, family = \
		cpuinfo._parse_cpu_brand_string_dx("Intel(R) Core(TM) i7 CPU         870  @ 2.93GHz")
		self.assertEqual("Intel(R) Core(TM) i7 CPU         870  @ 2.93GHz", brand)
		self.assertEqual((hz, scale), ('2.93', 9))
		self.assertEqual((vendor_id, stepping, model, family), (None, None, None, None))

		hz, scale, brand, vendor_id, stepping, model, family = \
		cpuinfo._parse_cpu_brand_string_dx("Intel(R) Pentium(R) CPU G640 @ 2.80GHz (2793.73-MHz K8-class CPU)")
		self.assertEqual("Intel(R) Pentium(R) CPU G640 @ 2.80GHz", brand)
		self.assertEqual((hz, scale), ('2.8', 9))
		self.assertEqual((vendor_id, stepping, model, family), (None, None, None, None))

		# NOTE: No @ symbol
		hz, scale, brand, vendor_id, stepping, model, family = \
		cpuinfo._parse_cpu_brand_string_dx("Intel(R) Pentium(R) D CPU 3.20GHz")
		self.assertEqual("Intel(R) Pentium(R) D CPU 3.20GHz", brand)
		self.assertEqual((hz, scale), ('3.2', 9))
		self.assertEqual((vendor_id, stepping, model, family), (None, None, None, None))

		# NOTE: No @ symbol and no Hz
		hz, scale, brand, vendor_id, stepping, model, family = \
		cpuinfo._parse_cpu_brand_string_dx("AMD Ryzen 7 2700X Eight-Core Processor          (3693.15-MHz K8-class CPU) (fam: 06, model: 2a, stepping: 07)")
		self.assertEqual("AMD Ryzen 7 2700X Eight-Core Processor", brand)
		self.assertEqual((hz, scale), ('3693.15', 6))
		self.assertEqual((vendor_id, stepping, model, family), (None, 7, 42, 6))





import unittest
from cpuinfo import *
import helpers


class MockDataSource(object):
	bits = '32bit'
	cpu_count = 8
	is_windows = False
	arch_string_raw = 'i86pc'
	uname_string_raw = 'i386'
	can_cpuid = False

	@staticmethod
	def has_isainfo():
		return True

	@staticmethod
	def has_kstat():
		return True

	@staticmethod
	def isainfo_vb():
		returncode = 0
		output = r'''
64-bit amd64 applications
	rdseed avx2 rdrand avx xsave pclmulqdq aes movbe sse4.2 sse4.1
	ssse3 amd_lzcnt popcnt amd_sse4a tscp ahf cx16 sse3 sse2 sse fxsr
amd_mmx mmx cmov amd_sysc cx8 tsc fpu

'''
		return returncode, output

	@staticmethod
	def kstat_m_cpu_info():
		returncode = 0
		output = r'''
module: cpu_info                        instance: 0
name:   cpu_info0                       class:    misc
	brand                           AMD Ryzen 7 2700X Eight-Core Processor
	cache_id                        0
	chip_id                         0
	clock_MHz                       3693
	clog_id                         0
	core_id                         0
	cpu_type                        i386
	crtime                          22.539390752
	current_clock_Hz                3692643590
	current_cstate                  1
	family                          23
	fpu_type                        i387 compatible
	implementation                  x86 (chipid 0x0 AuthenticAMD 800F82 family 23 model 8 step 2 clock 3693 MHz)
	model                           8
	ncore_per_chip                  8
	ncpu_per_chip                   8
	pg_id                           1
	pkg_core_id                     0
	snaptime                        120.971135132
	socket_type                     Unknown
	state                           on-line
	state_begin                     1553482276
	stepping                        2
	supported_frequencies_Hz        3692643590
	supported_max_cstates           0
	vendor_id                       AuthenticAMD


'''
		return returncode, output



class TestOpenIndiana_5_11_Ryzen_7(unittest.TestCase):
	def setUp(self):
		helpers.backup_data_source(cpuinfo)
		helpers.monkey_patch_data_source(cpuinfo, MockDataSource)

	def tearDown(self):
		helpers.restore_data_source(cpuinfo)

	'''
	Make sure calls return the expected number of fields.
	'''
	def test_returns(self):
		self.assertEqual(0, len(cpuinfo._get_cpu_info_from_registry()))
		self.assertEqual(0, len(cpuinfo._get_cpu_info_from_cpufreq_info()))
		self.assertEqual(0, len(cpuinfo._get_cpu_info_from_lscpu()))
		self.assertEqual(0, len(cpuinfo._get_cpu_info_from_proc_cpuinfo()))
		self.assertEqual(0, len(cpuinfo._get_cpu_info_from_sysctl()))
		self.assertEqual(10, len(cpuinfo._get_cpu_info_from_kstat()))
		self.assertEqual(0, len(cpuinfo._get_cpu_info_from_dmesg()))
		self.assertEqual(0, len(cpuinfo._get_cpu_info_from_cat_var_run_dmesg_boot()))
		self.assertEqual(0, len(cpuinfo._get_cpu_info_from_ibm_pa_features()))
		self.assertEqual(0, len(cpuinfo._get_cpu_info_from_sysinfo()))
		self.assertEqual(0, len(cpuinfo._get_cpu_info_from_cpuid()))
		self.assertEqual(17, len(cpuinfo._get_cpu_info_internal()))

	def test_get_cpu_info_from_kstat(self):
		info = cpuinfo._get_cpu_info_from_kstat()

		self.assertEqual('AuthenticAMD', info['vendor_id_raw'])
		self.assertEqual('AMD Ryzen 7 2700X Eight-Core Processor', info['brand_raw'])
		self.assertEqual('3.6930 GHz', info['hz_advertised_friendly'])
		self.assertEqual('3.6926 GHz', info['hz_actual_friendly'])
		self.assertEqual((3693000000, 0), info['hz_advertised'])
		self.assertEqual((3692643590, 0), info['hz_actual'])

		self.assertEqual(2, info['stepping'])
		self.assertEqual(8, info['model'])
		self.assertEqual(23, info['family'])
		self.assertEqual(
			['amd_mmx', 'amd_sysc', 'cmov', 'cx8', 'fpu', 'mmx', 'tsc']
			,
			info['flags']
		)

	def test_all(self):
		info = cpuinfo._get_cpu_info_internal()

		self.assertEqual('AuthenticAMD', info['vendor_id_raw'])
		self.assertEqual('AMD Ryzen 7 2700X Eight-Core Processor', info['brand_raw'])
		self.assertEqual('3.6930 GHz', info['hz_advertised_friendly'])
		self.assertEqual('3.6926 GHz', info['hz_actual_friendly'])
		self.assertEqual((3693000000, 0), info['hz_advertised'])
		self.assertEqual((3692643590, 0), info['hz_actual'])
		self.assertEqual('X86_32', info['arch'])
		self.assertEqual(32, info['bits'])
		self.assertEqual(8, info['count'])

		self.assertEqual('i86pc', info['arch_string_raw'])

		self.assertEqual(2, info['stepping'])
		self.assertEqual(8, info['model'])
		self.assertEqual(23, info['family'])
		self.assertEqual(
			['amd_mmx', 'amd_sysc', 'cmov', 'cx8', 'fpu', 'mmx', 'tsc']
			,
			info['flags']
		)
